"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createAggregationAutocompleter = void 0;
const mongodb_constants_1 = require("@mongodb-js/mongodb-constants");
const autocompleter_1 = require("../autocompleter");
const ace_compat_autocompleter_1 = require("./ace-compat-autocompleter");
const ace_compat_autocompleter_2 = require("./ace-compat-autocompleter");
const stage_autocompleter_1 = require("./stage-autocompleter");
const StageOperatorNames = new Set(mongodb_constants_1.STAGE_OPERATOR_NAMES);
function* parents(token) {
    let parent = token;
    while ((parent = parent.parent)) {
        yield parent;
    }
}
function removeQuotes(str) {
    return str.replace(/(^('|")|('|")$)/g, '');
}
function isTokenEqual(a, b) {
    return a.from === b.from && a.to === b.to;
}
function getPropertyNameFromPropertyToken(editorState, propertyToken) {
    if (!propertyToken.firstChild) {
        return '';
    }
    return removeQuotes(getTokenText(editorState, propertyToken.firstChild));
}
function padLines(str, pad = '  ') {
    return str
        .split('\n')
        .map((line) => `${pad}${line}`)
        .join('\n');
}
function getTokenText(editorState, token) {
    return editorState.sliceDoc(token.from, token.to);
}
function getStageNameForToken(editorState, token) {
    for (const parent of parents(token)) {
        if (parent.name === 'Property') {
            const propertyName = getPropertyNameFromPropertyToken(editorState, parent);
            if (parent.firstChild &&
                !isTokenEqual(parent.firstChild, token) &&
                StageOperatorNames.has(propertyName)) {
                return propertyName;
            }
        }
    }
    return null;
}
function createAggregationAutocompleter(options = {}) {
    const stageAutocompletions = (0, autocompleter_1.completer)('', { ...options, meta: ['stage'] });
    return (0, ace_compat_autocompleter_2.createAceCompatAutocompleter)({
        IdentifierLike({ context, prefix, token }) {
            var _a;
            const stageOperator = getStageNameForToken(context.state, token);
            if (stageOperator) {
                return (0, stage_autocompleter_1.createStageAutocompleter)({ stageOperator, ...options })(context);
            }
            const isInsideBlock = token.name === 'PropertyName' || ((_a = token.parent) === null || _a === void 0 ? void 0 : _a.name) === 'Property';
            return (0, ace_compat_autocompleter_1.createCompletionResultForIdPrefix)({
                prefix,
                completions: stageAutocompletions.map((completion) => {
                    const opName = completion.value;
                    return {
                        ...completion,
                        ...(completion.snippet && {
                            snippet: !isInsideBlock
                                ? `{\n${padLines(`${opName}: ${completion.snippet}`)}\n}`
                                : `${opName}: ${completion.snippet}`,
                        }),
                    };
                }),
            });
        },
    });
}
exports.createAggregationAutocompleter = createAggregationAutocompleter;
//# sourceMappingURL=aggregation-autocompleter.js.map