"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAncestryOfToken = exports.resolveTokenAtCursor = exports.completeWordsInString = void 0;
const autocomplete_1 = require("@codemirror/autocomplete");
const language_1 = require("@codemirror/language");
exports.completeWordsInString = (0, autocomplete_1.ifIn)(['String'], autocomplete_1.completeAnyWord);
function resolveTokenAtCursor(context) {
    return (0, language_1.syntaxTree)(context.state).resolveInner(context.pos, -1);
}
exports.resolveTokenAtCursor = resolveTokenAtCursor;
const getAncestorList = (node, doc) => {
    const ancestors = [];
    if (!(node === null || node === void 0 ? void 0 : node.parent)) {
        return ancestors;
    }
    if (['ArrayExpression', 'Array'].includes(node.parent.name)) {
        let prevSibling = node.prevSibling;
        let index = 0;
        while (prevSibling) {
            if (![':', ',', '['].includes(prevSibling.name)) {
                index++;
            }
            prevSibling = prevSibling.prevSibling;
        }
        ancestors.push(`[${index}]`);
    }
    if (node.name === 'Property' && node.firstChild) {
        const { from, to } = node.firstChild;
        const ancestor = doc.slice(from, to).replace(/"/g, '').replace(/'/g, '');
        ancestors.push(ancestor);
    }
    return ancestors.concat(getAncestorList(node.parent, doc));
};
function getAncestryOfToken(token, document) {
    var _a, _b, _c;
    const isAutocompletingPropertyName = ['ObjectExpression', 'PropertyDefinition'].includes(token.name) ||
        (((_b = (_a = token.parent) === null || _a === void 0 ? void 0 : _a.parent) === null || _b === void 0 ? void 0 : _b.name) === 'ObjectExpression' && !token.prevSibling);
    const list = getAncestorList(isAutocompletingPropertyName ? (_c = token.parent) === null || _c === void 0 ? void 0 : _c.parent : token, document).reverse();
    return list;
}
exports.getAncestryOfToken = getAncestryOfToken;
//# sourceMappingURL=utils.js.map